﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Security;
using System.Text;
using System.Windows;
using DaveSexton.SettingsSwitcher.Vsix.Properties;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.Win32;
using Ole = Microsoft.VisualStudio.OLE.Interop;

namespace DaveSexton.SettingsSwitcher.Vsix
{
	/// <summary>
	/// Implements the package exposed by this assembly.
	/// </summary>
	/// <remarks>
	/// The minimum requirement for a class to be considered a valid package for Visual Studio
	/// is to implement the IVsPackage interface and register itself with the shell.
	/// This package uses the helper classes defined inside the Managed Package Framework (MPF)
	/// to do it: it derives from the Package class that provides the implementation of the 
	/// IVsPackage interface and uses the registration attributes defined in the framework to 
	/// register itself and its components with the shell.
	/// </remarks>
	[PackageRegistration(UseManagedResourcesOnly = true)]		// This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is a package.
	[InstalledProductRegistration("#110", "#112", "1.0", IconResourceID = 400)]		// This attribute is used to register the information needed to show this package in the Help/About dialog of Visual Studio.
	[ProvideMenuResource("Menus.ctmenu", 1)]	// This attribute is needed to let the shell know that this package exposes some menus and toolbars.
	[ProvideOptionPage(typeof(OptionsDialogPage), "Settings Switcher", "General", 113, 114, supportsAutomation: true)]
	[Guid(Guids.vsixPkgString)]
	[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1506:AvoidExcessiveClassCoupling", Justification = "Time constraints.")]
	public sealed class SettingsSwitcherPackage : Package
	{
		public Uri SettingsDirectory
		{
			get
			{
				Contract.Ensures(Contract.Result<Uri>() == null || Contract.Result<Uri>().IsAbsoluteUri);
				Contract.Ensures(Contract.Result<Uri>() == null || Contract.Result<Uri>().IsFile);
				Contract.Ensures(Contract.Result<Uri>() == settingsDirectory);

				return settingsDirectory;
			}
		}

		private const string settingsFileExtension = ".vssettings";
		private const string settingsFilePattern = "*" + settingsFileExtension;
		private const string oldAutoSettingsFilePrefix = "Old.";

		private readonly FileSystemWatcher settingFilesWatcher = new FileSystemWatcher()
		{
			Filter = settingsFilePattern,
			IncludeSubdirectories = true,
			NotifyFilter = NotifyFilters.CreationTime | NotifyFilters.FileName | NotifyFilters.LastWrite
		};

		private EnvDTE.DTE Dte
		{
			get
			{
				Contract.Ensures(Contract.Result<EnvDTE.DTE>() != null);

				if (dte == null)
				{
					dte = (EnvDTE.DTE) GetService(typeof(SDTE));

					Contract.Assume(dte != null);
				}

				return dte;
			}
		}

		private OptionsDialogPage Options
		{
			get
			{
				Contract.Ensures(Contract.Result<OptionsDialogPage>() != null);

				if (options == null)
				{
					options = (OptionsDialogPage) GetDialogPage(typeof(OptionsDialogPage));

					Contract.Assume(options != null);
				}

				return options;
			}
		}

		private static readonly string SettingsOptionKey = typeof(SettingsSwitcherPackage).Name;

		private readonly object gate = new object();
		private readonly SettingsFileCollection settingsFiles = new SettingsFileCollection();
		private EnvDTE.DTE dte;
		private SettingsFile selectedSettingsFile;
		private Uri settingsDirectory, autoSettingsFile, oldAutoSettingsFile;
		private bool executingManageUserSettingsCommand;
		private OptionsDialogPage options;

		/// <summary>
		/// Default constructor of the package.
		/// Inside this method you can place any initialization code that does not require 
		/// any Visual Studio service because at this point the package object is created but 
		/// not sited yet inside Visual Studio environment. The place to do all the other 
		/// initialization is the Initialize method.
		/// </summary>
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2000:Dispose objects before losing scope", Justification = "The settingFilesWatcher field is disposed in the Dispose method of this class.")]
		public SettingsSwitcherPackage()
		{
			Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering constructor for: {0}", this.ToString()));

			settingFilesWatcher.Changed += SettingsFileDeletedCreatedChangedOrRenamed;
			settingFilesWatcher.Created += SettingsFileDeletedCreatedChangedOrRenamed;
			settingFilesWatcher.Deleted += SettingsFileDeletedCreatedChangedOrRenamed;
			settingFilesWatcher.Renamed += SettingsFileDeletedCreatedChangedOrRenamed;
			settingFilesWatcher.Error += SettingFilesWatcherError;
		}

		[ContractInvariantMethod]
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic", Justification = "Required for code contracts.")]
		private void ObjectInvariant()
		{
			Contract.Invariant(settingsFiles != null);
			Contract.Invariant(settingFilesWatcher != null);
			Contract.Invariant(settingsDirectory == null || settingsDirectory.IsAbsoluteUri);
			Contract.Invariant(settingsDirectory == null || settingsDirectory.IsFile);
			Contract.Invariant(autoSettingsFile == null || autoSettingsFile.IsAbsoluteUri);
			Contract.Invariant(autoSettingsFile == null || autoSettingsFile.IsFile);
			Contract.Invariant((autoSettingsFile == null) == (settingsDirectory == null));
			Contract.Invariant(oldAutoSettingsFile == null || oldAutoSettingsFile.IsAbsoluteUri);
			Contract.Invariant(oldAutoSettingsFile == null || oldAutoSettingsFile.IsFile);
			Contract.Invariant((oldAutoSettingsFile == null) == (settingsDirectory == null));
		}

		/// <summary>
		/// Initialization of the package; this method is called right after the package is sited, so this is the place
		/// where you can put all the initialization code that rely on services provided by VisualStudio.
		/// </summary>
		protected override void Initialize()
		{
			Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering Initialize() of: {0}", this.ToString()));

			base.Initialize();

			AddOptionKey(SettingsOptionKey);

			ReadSettingsDirectory();

			if (settingsDirectory != null)
			{
				settingFilesWatcher.Path = settingsDirectory.LocalPath;
				settingFilesWatcher.EnableRaisingEvents = true;

				lock (gate)
				{
					LoadSettingsFiles();

					if (selectedSettingsFile == null)
					{
						selectedSettingsFile = settingsFiles.FindByFullName(Settings.Default.SelectedSettingsFile);
					}
				}
			}

			var menus = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;

			if (menus != null)
			{
				foreach (var command in InitializeCommands())
				{
					menus.AddCommand(command);
				}
			}

			InitializePriorityCommands();

			Contract.Assert(settingsDirectory == null || settingsDirectory.IsAbsoluteUri);
			Contract.Assert(settingsDirectory == null || settingsDirectory.IsFile);
			Contract.Assert(autoSettingsFile == null || autoSettingsFile.IsAbsoluteUri);
			Contract.Assert(autoSettingsFile == null || autoSettingsFile.IsFile);
			Contract.Assert(oldAutoSettingsFile == null || oldAutoSettingsFile.IsAbsoluteUri);
			Contract.Assert(oldAutoSettingsFile == null || oldAutoSettingsFile.IsFile);
		}

		private IEnumerable<MenuCommand> InitializeCommands()
		{
			Contract.Ensures(Contract.Result<IEnumerable<MenuCommand>>() != null);
			Contract.Ensures(settingsDirectory == Contract.OldValue(settingsDirectory));
			Contract.Ensures(autoSettingsFile == Contract.OldValue(autoSettingsFile));
			Contract.Ensures(oldAutoSettingsFile == Contract.OldValue(oldAutoSettingsFile));

			var switcherCommand = new OleMenuCommand(
				SettingsSelector,
				new CommandID(Guids.VsixCmdSet, (int) PackageCommandIds.SettingsSelector));

			switcherCommand.BeforeQueryStatus += ImportSelectedSettingsQueryStatus;

			yield return switcherCommand;

			yield return new OleMenuCommand(
				SettingsSelectorGetItems,
				new CommandID(Guids.VsixCmdSet, (int) PackageCommandIds.SettingsSelectorGetItems));

			var exportCommand = new OleMenuCommand(
				ExportCurrentSettings,
				new CommandID(Guids.VsixCmdSet, (int) PackageCommandIds.ExportCurrentSettings));

			exportCommand.BeforeQueryStatus += ExportCurrentSettingsQueryStatus;

			yield return exportCommand;

			var formatSolutionCommand = new OleMenuCommand(
				FormatSolution,
				new CommandID(Guids.VsixCmdSet, (int) PackageCommandIds.FormatSolution));

			formatSolutionCommand.BeforeQueryStatus += FormatSolutionQueryStatus;

			yield return formatSolutionCommand;
		}

		private void InitializePriorityCommands()
		{
			var priority = GetService(typeof(SVsRegisterPriorityCommandTarget)) as IVsRegisterPriorityCommandTarget;

			if (priority != null)
			{
				uint cookie;
				ErrorHandler.ThrowOnFailure(priority.RegisterPriorityCommandTarget(
					0,
					new AnonymousCommandTarget(
						(groupId, commands, text) => (int) Ole.Constants.OLECMDERR_E_UNKNOWNGROUP,
						(groupId, commandId, opts, input, output) =>
						{
							if (commandId == (uint) VSConstants.VSStd2KCmdID.ManageUserSettings
								&& groupId == Guids.VsStd2kCmdId
								&& !executingManageUserSettingsCommand)
							{
								var result = VsShellUtilities.ShowMessageBox(
									this,
									Resources.ImportAndExportCommandIsIncompatible,
									Resources.ImportAndExportCommandIsIncompatibleTitle,
									OLEMSGICON.OLEMSGICON_WARNING,
									OLEMSGBUTTON.OLEMSGBUTTON_OKCANCEL,
									OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_SECOND);

								ErrorHandler.ThrowOnFailure(result);

								if (result == (int) VSConstants.MessageBoxResult.IDCANCEL)
								{
									return (int) VSConstants.S_OK;
								}
							}

							return (int) Ole.Constants.OLECMDERR_E_NOTSUPPORTED;
						}),
					out cookie));
			}
		}

		private void ReadSettingsDirectory()
		{
			var page = Dte.Properties["Environment", "Import and Export Settings"];

			Contract.Assume(page != null);

			var autoSaveFileProperty = page.Item("AutoSaveFile");

			Contract.Assume(autoSaveFileProperty != null);

			var autoSaveFile = autoSaveFileProperty.Value as string;

			Contract.Assume(autoSaveFile != null);

			var settingsDirectoryPath = Path.GetDirectoryName(autoSaveFile);

			settingsDirectory = new Uri(settingsDirectoryPath, UriKind.Absolute);
			oldAutoSettingsFile = new Uri(Path.Combine(settingsDirectoryPath, oldAutoSettingsFilePrefix + Path.GetFileName(autoSaveFile)));
			autoSettingsFile = new Uri(autoSaveFile, UriKind.Absolute);

			Contract.Assume(settingsDirectory.IsAbsoluteUri);
			Contract.Assume(settingsDirectory.IsFile);
			Contract.Assume(autoSettingsFile.IsAbsoluteUri);
			Contract.Assume(autoSettingsFile.IsFile);
			Contract.Assume(oldAutoSettingsFile.IsAbsoluteUri);
			Contract.Assume(oldAutoSettingsFile.IsFile);

			// TODO: Special case team settings file too.
		}

		private void LoadSettingsFiles()
		{
			Contract.Requires(SettingsDirectory != null);

			Contract.Assert(autoSettingsFile != null);

			settingsFiles.Clear();

			var previousSelection = selectedSettingsFile;

			selectedSettingsFile = null;

			var nameManager = new SettingsFileNameManager(autoSettingsFile, oldAutoSettingsFile);

			try
			{
				var files = Directory.EnumerateFiles(settingsDirectory.LocalPath, settingsFilePattern, SearchOption.AllDirectories);

				Contract.Assume(files != null);

				foreach (var settingsFile in from file in files
																		 where !string.Equals(file, autoSettingsFile.LocalPath, StringComparison.OrdinalIgnoreCase)
																		 where !string.Equals(file, oldAutoSettingsFile.LocalPath, StringComparison.OrdinalIgnoreCase)
																		 let settingsFile = new SettingsFile(new Uri(file, UriKind.Absolute), settingsDirectory, nameManager)
																		 orderby settingsFile.Name
																		 select settingsFile)
				{
					settingsFiles.Add(settingsFile);
				}

				if (File.Exists(autoSettingsFile.LocalPath))
				{
					Contract.Assume(settingsDirectory.IsAbsoluteUri);
					Contract.Assume(settingsDirectory.IsFile);
					Contract.Assume(autoSettingsFile.IsAbsoluteUri);
					Contract.Assume(autoSettingsFile.IsFile);

					settingsFiles.Add(new SettingsFile(autoSettingsFile, settingsDirectory, nameManager));
				}

				if (File.Exists(oldAutoSettingsFile.LocalPath))
				{
					Contract.Assume(settingsDirectory.IsAbsoluteUri);
					Contract.Assume(settingsDirectory.IsFile);
					Contract.Assume(oldAutoSettingsFile.IsAbsoluteUri);
					Contract.Assume(oldAutoSettingsFile.IsFile);

					settingsFiles.Add(new SettingsFile(oldAutoSettingsFile, settingsDirectory, nameManager));
				}
			}
			catch (IOException ex)
			{
				Debug.WriteLine("Failed to iterate files: " + ex, "Settings Switcher");
			}
			catch (SecurityException ex)
			{
				Debug.WriteLine("Failed to iterate files: " + ex, "Settings Switcher");
			}

			if (previousSelection != null)
			{
				selectedSettingsFile = settingsFiles.FirstOrDefault(file => file.FullPath == previousSelection.FullPath);
			}

			Contract.Assume(settingsDirectory.IsAbsoluteUri);
			Contract.Assume(settingsDirectory.IsFile);
			Contract.Assume(autoSettingsFile.IsAbsoluteUri);
			Contract.Assume(autoSettingsFile.IsFile);
			Contract.Assume(oldAutoSettingsFile.IsAbsoluteUri);
			Contract.Assume(oldAutoSettingsFile.IsFile);
		}

		private void RefreshCommands()
		{
			var shell = (IVsUIShell) GetService(typeof(SVsUIShell));

			Contract.Assume(shell != null);

			ErrorHandler.ThrowOnFailure(shell.UpdateCommandUI(VSConstants.S_FALSE));
		}

		protected override void OnLoadOptions(string key, Stream stream)
		{
			Contract.Assume(key == SettingsOptionKey);
			Contract.Assume(stream != null);

			if (Options.SynchronizeSelectedSettingsWithSolution)
			{
				var reader = new StreamReader(stream, Encoding.UTF8);

				var path = reader.ReadToEnd();

				if (!string.IsNullOrEmpty(path))
				{
					lock (gate)
					{
						var file = settingsFiles.FindByFullName(path);

						if (file != null && file != selectedSettingsFile)
						{
							selectedSettingsFile = file;

							ImportSelectedSettings(saveSelectedAsDefault: false);
						}
					}

					RefreshCommands();
				}
			}

			base.OnLoadOptions(key, stream);
		}

		protected override void OnSaveOptions(string key, Stream stream)
		{
			Contract.Assume(key == SettingsOptionKey);
			Contract.Assume(stream != null);

			if (Options.SynchronizeSelectedSettingsWithSolution)
			{
				var file = selectedSettingsFile;

				if (file != null)
				{
					var writer = new StreamWriter(stream, Encoding.UTF8);

					writer.Write(file.FullPath.LocalPath);
					writer.Flush();
				}
			}

			base.OnSaveOptions(key, stream);
		}

		private void ExecuteManageUserSettingsCommand(string arguments)
		{
			Contract.Requires(arguments != null);
			Contract.Ensures(settingsDirectory == Contract.OldValue(settingsDirectory));

			try
			{
				executingManageUserSettingsCommand = true;

				Contract.Assume(Dte.Commands != null);

				Dte.Commands.Raise(Guids.vsStd2kCmdIdString, (int) VSConstants.VSStd2KCmdID.ManageUserSettings, arguments, null);
			}
			finally
			{
				executingManageUserSettingsCommand = false;
			}
		}

		private void ImportSelectedSettings(bool saveSelectedAsDefault = true)
		{
			if (selectedSettingsFile != null)
			{
				ExecuteManageUserSettingsCommand("-import:\"" + selectedSettingsFile.FullPath.LocalPath + "\"");

				if (saveSelectedAsDefault)
				{
					Contract.Assume(selectedSettingsFile != null);

					Settings.Default.SelectedSettingsFile = selectedSettingsFile.FullPath.LocalPath;
				}
			}
			else if (saveSelectedAsDefault)
			{
				Settings.Default.SelectedSettingsFile = null;
			}

			if (saveSelectedAsDefault)
			{
				Settings.Default.Save();
			}
		}

		private void ExportCurrentSettings(object sender, EventArgs e)
		{
			Contract.Requires(settingsDirectory != null);

			if (Options.ExportAutomaticallyOverwritesSelectedSettingsFile
				&& selectedSettingsFile != null
				&& selectedSettingsFile.FullPath != autoSettingsFile
				&& selectedSettingsFile.FullPath != oldAutoSettingsFile)
			{
				ExportCurrentSettings(selectedSettingsFile.FullPath.LocalPath);
			}
			else
			{
				var dialog = new SaveFileDialog()
				{
					AddExtension = true,
					DefaultExt = settingsFileExtension,
					Filter = Resources.SettingsFilesGroupName + " (" + settingsFilePattern + ")|" + settingsFilePattern,
					InitialDirectory = settingsDirectory.LocalPath,
					OverwritePrompt = true,
					Title = Resources.ExportCurrentSettingsDialogTitle
				};

				if (dialog.ShowDialog(Application.Current.MainWindow) ?? false)
				{
					ExportCurrentSettings(dialog.FileName);
				}
			}
		}

		private void ExportCurrentSettings(string file)
		{
			Contract.Requires(settingsDirectory != null);
			Contract.Requires(file != null);

			ExecuteManageUserSettingsCommand("-export:\"" + file + "\"");

			lock (gate)
			{
				selectedSettingsFile = null;

				LoadSettingsFiles();

				selectedSettingsFile = settingsFiles.FindByFullName(file);
			}

			RefreshCommands();
		}

		private void ImportSelectedSettingsQueryStatus(object sender, EventArgs e)
		{
			Contract.Requires(sender != null);

			var command = (OleMenuCommand) sender;

			command.Enabled = settingsDirectory != null;
		}

		private void ExportCurrentSettingsQueryStatus(object sender, EventArgs e)
		{
			Contract.Requires(sender != null);

			var command = (OleMenuCommand) sender;

			command.Enabled = settingsDirectory != null;
		}

		private void FormatSolutionQueryStatus(object sender, EventArgs e)
		{
			Contract.Requires(sender != null);

			var command = (MenuCommand) sender;

			command.Enabled = Dte.Solution != null && Dte.Solution.IsOpen;
		}

		[ContractVerification(false)]
		private void FormatSolution(object sender, EventArgs e)
		{
			var solution = Dte.Solution;

			if (solution != null
				&& solution.IsOpen
				&& ShouldFormatSolution(solution))
			{
				var projects = solution.Projects;

				Contract.Assume(projects != null);

				foreach (EnvDTE.Project project in projects)
				{
					if (project != null)
					{
						FormatSolutionRecursive(project.ProjectItems);
					}
				}
			}
		}

		private bool ShouldFormatSolution(EnvDTE.Solution solution)
		{
			Contract.Requires(solution != null);

			var sourceControl = Dte.SourceControl;

			int result;

			if (sourceControl != null && sourceControl.IsItemUnderSCC(solution.FullName))
			{
				result = VsShellUtilities.ShowMessageBox(
					this,
					Resources.ConfirmFormatSourceControlledSolution,
					Resources.ConfirmFormatSolutionTitle,
					OLEMSGICON.OLEMSGICON_QUERY,
					OLEMSGBUTTON.OLEMSGBUTTON_OKCANCEL,
					OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_SECOND);
			}
			else
			{
				result = VsShellUtilities.ShowMessageBox(
					this,
					Resources.ConfirmFormatSolution,
					Resources.ConfirmFormatSolutionTitle,
					OLEMSGICON.OLEMSGICON_QUERY,
					OLEMSGBUTTON.OLEMSGBUTTON_OKCANCEL,
					OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_SECOND);
			}

			ErrorHandler.ThrowOnFailure(result);

			return result == (int) VSConstants.MessageBoxResult.IDOK;
		}

		[ContractVerification(false)]
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes", Justification = "It's safe to bail out here for any catchable exception.")]
		private void FormatSolutionRecursive(EnvDTE.ProjectItems projectItems)
		{
			if (projectItems != null && projectItems.Count > 0)
			{
				try
				{
					foreach (EnvDTE.ProjectItem item in projectItems)
					{
						if (item != null)
						{
							if (item.Kind == VSConstants.ItemTypeGuid.PhysicalFile_string
								&& item.FileCodeModel != null)
							{
								FormatDocument(item);
							}

							FormatSolutionRecursive(item.ProjectItems);
						}
					}
				}
				catch (Exception ex)
				{
					Debug.WriteLine("Failed to iterate items in project.  Error: " + ex, "Settings Switcher");
				}
			}
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes", Justification = "It's safe to bail out here for any catchable exception.")]
		private void FormatDocument(EnvDTE.ProjectItem item)
		{
			Contract.Requires(item != null);

			try
			{
				var fileName = item.FileNames[0];

				Contract.Assume(Dte.ItemOperations != null);

				var shouldClose = fileName == null || !Dte.ItemOperations.IsFileOpen(fileName, VSConstants.LOGVIEWID.Code_string);

				var window = item.Open(VSConstants.LOGVIEWID.Code_string);

				if (window != null)
				{
					window.Activate();

					Contract.Assume(Dte.Commands != null);

					Dte.Commands.Raise(Guids.vsStd2kCmdIdString, (int) VSConstants.VSStd2KCmdID.FORMATDOCUMENT, null, null);

					if (shouldClose)
					{
						window.Close(EnvDTE.vsSaveChanges.vsSaveChangesYes);
					}
					else
					{
						item.Save();
					}
				}
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Failed to format item.  Error: " + ex, "Settings Switcher");
			}
		}

		private void SettingsSelector(object sender, EventArgs e)
		{
			var args = e as OleMenuCmdEventArgs;

			if (args != null)
			{
				var input = args.InValue;
				var output = args.OutValue;

				if (input != null)
				{
					lock (gate)
					{
						selectedSettingsFile = settingsFiles[input as string];

						ImportSelectedSettings();
					}
				}
				else if (output != IntPtr.Zero)
				{
					var file = selectedSettingsFile;

					if (file == null)
					{
						Marshal.GetNativeVariantForObject(null, output);
					}
					else
					{
						Marshal.GetNativeVariantForObject(file.DisplayName, output);
					}
				}
			}
		}

		private void SettingsSelectorGetItems(object sender, EventArgs e)
		{
			var args = e as OleMenuCmdEventArgs;

			if (args != null)
			{
				var output = args.OutValue;

				if (output != IntPtr.Zero)
				{
					Marshal.GetNativeVariantForObject(settingsFiles.DisplayNames, output);
				}
			}
		}

		private void SettingsFileDeletedCreatedChangedOrRenamed(object sender, EventArgs e)
		{
			lock (gate)
			{
				if (settingsDirectory != null)
				{
					LoadSettingsFiles();
				}
			}

			RefreshCommands();
		}

		private void SettingFilesWatcherError(object sender, ErrorEventArgs e)
		{
			Debug.Fail("Settings Switcher - File Watcher Error", e.GetException().ToString());
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				settingFilesWatcher.Dispose();
			}

			base.Dispose(disposing);
		}
	}
}