﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Globalization;
using System.IO;
using System.Linq;
using DaveSexton.SettingsSwitcher.Properties;

namespace DaveSexton.SettingsSwitcher
{
	/// <summary>
	/// Generates unique display names for settings files.
	/// </summary>
	public sealed class SettingsFileNameManager
	{
#if DEBUG
		internal bool IsSealed
		{
			get
			{
				Contract.Ensures(Contract.Result<bool>() == isSealed);

				return isSealed;
			}
		}
#endif

		private readonly List<SettingsFile> files = new List<SettingsFile>();
		private readonly Uri autoSettingsFile, oldAutoSettingsFile;
#if DEBUG
		private bool isSealed;
#endif

		/// <summary>
		/// Constructs a new instance of the <see cref="SettingsFileNameManager"/> class.
		/// </summary>
		/// <param name="autoSettingsFile">The path to the auto-save settings file.  The file does not have to exist.</param>
		/// <param name="oldAutoSettingsFile">The path to the old auto-save settings file.  The file does not have to exist.</param>
		public SettingsFileNameManager(Uri autoSettingsFile, Uri oldAutoSettingsFile)
		{
			Contract.Requires(autoSettingsFile != null);
			Contract.Requires(autoSettingsFile.IsAbsoluteUri);
			Contract.Requires(autoSettingsFile.IsFile);
			Contract.Requires(oldAutoSettingsFile != null);
			Contract.Requires(oldAutoSettingsFile.IsAbsoluteUri);
			Contract.Requires(oldAutoSettingsFile.IsFile);

			this.autoSettingsFile = autoSettingsFile;
			this.oldAutoSettingsFile = oldAutoSettingsFile;
		}

		[ContractInvariantMethod]
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic", Justification = "Required for code contracts.")]
		private void ObjectInvariant()
		{
			Contract.Invariant(files != null);
			Contract.Invariant(autoSettingsFile != null);
			Contract.Invariant(autoSettingsFile.IsAbsoluteUri);
			Contract.Invariant(autoSettingsFile.IsFile);
			Contract.Invariant(oldAutoSettingsFile != null);
			Contract.Invariant(oldAutoSettingsFile.IsAbsoluteUri);
			Contract.Invariant(oldAutoSettingsFile.IsFile);
		}

		internal void AddFile(SettingsFile file)
		{
#if DEBUG
			Contract.Requires(!IsSealed);
#endif
			Contract.Requires(file != null);

			files.Add(file);
		}

		internal string GetDisplayName(SettingsFile file)
		{
			Contract.Requires(file != null);
			Contract.Ensures(Contract.Result<string>() != null);

#if DEBUG
			isSealed = true;
#endif

			/* Display names must be unique.  It seems that Visual Studio can only display strings in 
			 * combo commands; therefore, the display name is used to find the selected object.
			 */

			string format;

			if (file.FullPath == autoSettingsFile || file.FullPath == oldAutoSettingsFile)
			{
				format = Resources.SettingsFileAutoSavedDisplayNameFormat;
			}
			else if (HasFileWithSameName(file))
			{
				format = Resources.SettingsFileRelativeDisplayNameFormat;
			}
			else
			{
				format = Resources.SettingsFileDefaultDisplayNameFormat;
			}

			return FormatName(file, format);
		}

		private static string FormatName(SettingsFile file, string format)
		{
			Contract.Requires(file != null);
			Contract.Requires(format != null);
			Contract.Ensures(Contract.Result<string>() != null);

			return string.Format(CultureInfo.CurrentCulture, format, file.Name, Path.GetDirectoryName(file.RelativePath.ToString()));
		}

		private bool HasFileWithSameName(SettingsFile file)
		{
			Contract.Requires(file != null);

			return (from other in files
							where other != file
								 && string.Equals(other.Name, file.Name, StringComparison.OrdinalIgnoreCase)
							select other)
							.Any();
		}
	}
}